<?php

/**
 * Copyright distrib (2018)
 *
 * contact@opendistrib.net
 *
 * Ce logiciel est un programme informatique servant à aider les producteurs
 * à distribuer leur production en circuits courts.
 *
 * Ce logiciel est régi par la licence CeCILL soumise au droit français et
 * respectant les principes de diffusion des logiciels libres. Vous pouvez
 * utiliser, modifier et/ou redistribuer ce programme sous les conditions
 * de la licence CeCILL telle que diffusée par le CEA, le CNRS et l'INRIA
 * sur le site "http://www.cecill.info".
 *
 * En contrepartie de l'accessibilité au code source et des droits de copie,
 * de modification et de redistribution accordés par cette licence, il n'est
 * offert aux utilisateurs qu'une garantie limitée.  Pour les mêmes raisons,
 * seule une responsabilité restreinte pèse sur l'auteur du programme,  le
 * titulaire des droits patrimoniaux et les concédants successifs.
 *
 * A cet égard  l'attention de l'utilisateur est attirée sur les risques
 * associés au chargement,  à l'utilisation,  à la modification et/ou au
 * développement et à la reproduction du logiciel par l'utilisateur étant
 * donné sa spécificité de logiciel libre, qui peut le rendre complexe à
 * manipuler et qui le réserve donc à des développeurs et des professionnels
 * avertis possédant  des  connaissances  informatiques approfondies.  Les
 * utilisateurs sont donc invités à charger  et  tester  l'adéquation  du
 * logiciel à leurs besoins dans des conditions permettant d'assurer la
 * sécurité de leurs systèmes et ou de leurs données et, plus généralement,
 * à l'utiliser et l'exploiter dans les mêmes conditions de sécurité.
 *
 * Le fait que vous puissiez accéder à cet en-tête signifie que vous avez
 * pris connaissance de la licence CeCILL, et que vous en avez accepté les
 * termes.
 */

namespace common\logic\Distribution\Distribution;

use common\logic\Distribution\PointSaleDistribution\PointSaleDistribution;
use common\logic\Distribution\ProductDistribution\ProductDistribution;
use common\logic\Order\Order\Order;
use common\logic\Producer\Producer\Producer;
use common\helpers\GlobalParam;
use common\components\ActiveRecordCommon;
use common\logic\Product\Product\Product;

/**
 * This is the model class for table "distribution".
 *
 */
class Distribution extends ActiveRecordCommon
{
    /**
     * @inheritdoc
     */
    public static function tableName()
    {
        return 'distribution';
    }

    /**
     * @inheritdoc
     */
    public function rules()
    {
        return [
            [['date'], 'required'],
            [['active'], 'integer'],
            [['date'], 'safe'],
        ];
    }

    /**
     * @inheritdoc
     */
    public function attributeLabels()
    {
        return [
            'id' => 'ID',
            'date' => 'Date',
            'active' => 'Actif',
        ];
    }

    /*
     * Relations
     */

    public function getProducer()
    {
        return $this->hasOne(Producer::class, ['id' => 'id_producer']);
    }

    public function getOrder()
    {
        return $this->hasMany(Order::class, ['id_distribution' => 'id']);
    }

    public function getProductDistribution()
    {
        return $this->hasMany(ProductDistribution::class, ['id_distribution' => 'id']);
    }

    public function getPointSaleDistribution()
    {
        return $this->hasMany(PointSaleDistribution::class, ['id_distribution' => 'id']);
    }

    /**
     * Lie un produit aux jours de distribution futurs.
     *
     * @param Product $product
     */
    public static function linkProductIncomingDistributions($product)
    {
        $distributionsArray = self::getIncoming();
        foreach ($distributionsArray as $distribution) {
            $distribution->linkProduct($product);
        }
    }

    /**
     * Lie un point de vente aux jours de distribution futurs.
     *
     * @param PointSale $pointSale
     */
    public static function linkPointSaleIncomingDistributions($pointSale)
    {
        $distributionsArray = self::getIncoming();
        foreach ($distributionsArray as $distribution) {
            $distribution->linkPointSale($pointSale);
        }
    }

    /**
     *
     * @param type $pointSale
     */
    public function linkPointSale($pointSale)
    {
        $pointSaleDistribution = PointSaleDistribution::searchOne([
            'id_distribution' => $this->id,
            'id_point_sale' => $pointSale->id
        ]);

        if (!$pointSaleDistribution) {
            $pointSaleDistribution = new PointSaleDistribution();
            $pointSaleDistribution->id_distribution = $this->id;
            $pointSaleDistribution->id_point_sale = $pointSale->id;
        }

        $dayDistribution = date('N', strtotime($this->date));
        $daysArray = [
            1 => 'monday',
            2 => 'tuesday',
            3 => 'wednesday',
            4 => 'thursday',
            5 => 'friday',
            6 => 'saturday',
            7 => 'sunday',
        ];

        $pointSaleDistribution->delivery = 0;
        $deliveryDay = 'delivery_' . $daysArray[$dayDistribution];
        if ($pointSale->$deliveryDay) {
            $pointSaleDistribution->delivery = 1;
        }

        $pointSaleDistribution->save();
    }

    public function isPointSaleActive($distribution, $pointSaleId)
    {
        $pointSaleActive = false;

        if ($distribution->pointSaleDistribution) {
            foreach ($distribution->pointSaleDistribution as $pointSaleDistribution) {
                if ($pointSaleDistribution->id_point_sale == $pointSaleId && $pointSaleDistribution->delivery) {
                    $pointSaleActive = true;
                }
            }
        }

        return $pointSaleActive;
    }

    /**
     * Active ou désactive la distribution.
     */
    public function active($active = true)
    {
        PointSaleDistribution::setAll($this->id, true);
        $this->active = (int)$active;
        $this->save();

        if ($active) {
            // ajout des abonnements
             Subscription::addAll($this->date);
        }
    }

}
